@@ -3,8 +3,8 @@
 (c) The GRASP/AQUA Project, Glasgow University, 1992-1998
 -}
 
-{-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE DerivingVia #-}
 {-# LANGUAGE MagicHash #-}
 {-# LANGUAGE PatternSynonyms #-}
 {-# LANGUAGE UnboxedTuples #-}
@@ -15,7 +15,7 @@ module GHC.Types.Unique.Supply (
 
         -- ** Operations on supplies
         uniqFromSupply, uniqsFromSupply, -- basic ops
-        takeUniqFromSupply, uniqFromMask,
+        takeUniqFromSupply, uniqFromTag,
 
         mkSplitUniqSupply,
         splitUniqSupply, listSplitUniqSupply,
@@ -39,18 +39,27 @@ import GHC.IO
 
 import GHC.Utils.Monad
 import Control.Monad
-import Data.Char
+import Data.Word
 import GHC.Exts( Ptr(..), noDuplicate#, oneShot )
-#if MIN_VERSION_GLASGOW_HASKELL(9,1,0,0)
-import GHC.Exts( Int(..), word2Int#, fetchAddWordAddr#, plusWord#, readWordOffAddr# )
-#if defined(DEBUG)
-import GHC.Utils.Misc
-#endif
-#endif
 import Foreign.Storable
+import GHC.Utils.Monad.State.Strict as Strict
 
-#include "Unique.h"
-#include "HsVersions.h"
+#include "MachDeps.h"
+
+#if WORD_SIZE_IN_BITS != 64
+#define NO_FETCH_ADD
+#endif
+
+#if defined(NO_FETCH_ADD)
+import GHC.Exts ( atomicCasWord64Addr#, eqWord64#, readWord64OffAddr# )
+#else
+import GHC.Exts( fetchAddWordAddr#, word64ToWord# )
+#endif
+
+import GHC.Exts ( Addr#, State#, Word64#, RealWorld )
+import GHC.Int ( Int(..) )
+import GHC.Word( Word64(..) )
+import GHC.Exts( plusWord64#, int2Word#, wordToWord64# )
 
 {-
 ************************************************************************
@@ -65,17 +74,17 @@ import Foreign.Storable
 The basic idea (due to Lennart Augustsson) is that a UniqSupply is
 lazily-evaluated infinite tree.
 
-* At each MkSplitUniqSupply node is a unique Int, and two
+* At each MkSplitUniqSupply node is a unique Word64, and two
   sub-trees (see data UniqSupply)
 
 * takeUniqFromSupply :: UniqSupply -> (Unique, UniqSupply)
-  returns the unique Int and one of the sub-trees
+  returns the unique Word64 and one of the sub-trees
 
 * splitUniqSupply :: UniqSupply -> (UniqSupply, UniqSupply)
   returns the two sub-trees
 
 * When you poke on one of the thunks, it does a foreign call
-  to get a fresh Int from a thread-safe counter, and returns
+  to get a fresh Word64 from a thread-safe counter, and returns
   a fresh MkSplitUniqSupply node.  This has to be as efficient
   as possible: it should allocate only
      * The fresh node
@@ -87,25 +96,25 @@ The general design (used throughout GHC) is to:
 
 * For creating new uniques either a UniqSupply is used and threaded through
   or for monadic code a MonadUnique instance might conjure up uniques using
-  `uniqFromMask`.
+  `uniqFromTag`.
 * Different parts of the compiler will use a UniqSupply or MonadUnique instance
-  with a specific mask. This way the different parts of the compiler will
-  generate uniques with different masks.
+  with a specific tag. This way the different parts of the compiler will
+  generate uniques with different tags.
 
-If different code shares the same mask then care has to be taken that all uniques
+If different code shares the same tag then care has to be taken that all uniques
 still get distinct numbers. Usually this is done by relying on genSym which
 has *one* counter per GHC invocation that is relied on by all calls to it.
 But using something like the address for pinned objects works as well and in fact is done
 for fast strings.
 
 This is important for example in the simplifier. Most passes of the simplifier use
-the same mask 's'. However in some places we create a unique supply using `mkSplitUniqSupply`
+the same tag 's'. However in some places we create a unique supply using `mkSplitUniqSupply`
 and thread it through the code, while in GHC.Core.Opt.Simplify.Monad  we use the
 `instance MonadUnique SimplM`, which uses `mkSplitUniqSupply` in getUniqueSupplyM
-and `uniqFromMask` in getUniqeM.
+and `uniqFromTag` in getUniqueM.
 
-Ultimately all these boil down to each new unique consisting of the mask and the result from
-a call to `genSym`. The later producing a distinct number for each invocation ensuring
+Ultimately all these boil down to each new unique consisting of the tag and the result from
+a call to `genSym`. The latter producing a distinct number for each invocation ensuring
 uniques are distinct.
 
 Note [Optimising the unique supply]
@@ -117,7 +126,7 @@ The inner loop of mkSplitUniqSupply is a function closure
         case unIO genSym s1 of { (# s2, u #) ->
         case unIO (unsafeDupableInterleaveIO (IO mk_supply)) s2 of { (# s3, x #) ->
         case unIO (unsafeDupableInterleaveIO (IO mk_supply)) s3 of { (# s4, y #) ->
-        (# s4, MkSplitUniqSupply (mask .|. u) x y #)
+        (# s4, MkSplitUniqSupply (tag .|. u) x y #)
         }}}}
 
 It's a classic example of an IO action that is captured and then called
@@ -131,7 +140,7 @@ We used to write it as:
                  genSym      >>= \ u ->
                  mk_supply   >>= \ s1 ->
                  mk_supply   >>= \ s2 ->
-                 return (MkSplitUniqSupply (mask .|. u) s1 s2)
+                 return (MkSplitUniqSupply (tag .|. u) s1 s2)
 
 and to rely on -fno-state-hack, full laziness and inlining to get the same
 result. It was very brittle and required enabling -fno-state-hack globally. So
@@ -150,29 +159,28 @@ The code for this is about as optimized as it gets, but we can't
 get around the need to allocate one `UniqSupply` for each Unique
 we need.
 
-For code in IO we can improve on this by threading only the *mask*
-we are going to use for Uniques. Using `uniqFromMask` to
+For code in IO we can improve on this by threading only the *tag*
+we are going to use for Uniques. Using `uniqFromTag` to
 generate uniques as needed. This gets rid of the overhead of
 allocating a new UniqSupply for each unique generated. It also avoids
-frequent state updates when the Unique/Mask is part of the state in a
+frequent state updates when the Unique/Tag is part of the state in a
 state monad.
 
-For monadic code in IO which always uses the same mask we can go further
-and hardcode the mask into the MonadUnique instance. On top of all the
-benefits of threading the mask this *also* has the benefit of avoiding
-the mask getting captured in thunks, or being passed around at runtime.
-It does however come at the cost of having to use a fixed Mask for all
-code run in this Monad. But rememeber, the Mask is purely cosmetic:
-See Note [Uniques and masks].
+For monadic code in IO which always uses the same tag we can go further
+and hardcode the tag into the MonadUnique instance. On top of all the
+benefits of threading the tag this *also* has the benefit of avoiding
+the tag getting captured in thunks, or being passed around at runtime.
+It does however come at the cost of having to use a fixed tag for all
+code run in this Monad. The tag is mostly cosmetic: See Note [Uniques and tags].
 
 NB: It's *not* an optimization to pass around the UniqSupply inside an
-IORef instead of the mask. While this would avoid frequent state updates
+IORef instead of the tag. While this would avoid frequent state updates
 it still requires allocating one UniqSupply per Unique. On top of some
 overhead for reading/writing to/from the IORef.
 
 All of this hinges on the assumption that UniqSupply and
-uniqFromMask use the same source of distinct numbers (`genSym`) which
-allows both to be used at the same time, with the same mask, while still
+uniqFromTag use the same source of distinct numbers (`genSym`) which
+allows both to be used at the same time, with the same tag, while still
 ensuring distinct uniques.
 One might consider this fact to be an "accident". But GHC worked like this
 as far back as source control history goes. It also allows the later two
@@ -188,18 +196,17 @@ optimizations to be used. So it seems safe to depend on this fact.
 -- also manufacture an arbitrary number of further 'UniqueSupply' values,
 -- which will be distinct from the first and from all others.
 data UniqSupply
-  = MkSplitUniqSupply {-# UNPACK #-} !Int -- make the Unique with this
+  = MkSplitUniqSupply {-# UNPACK #-} !Word64 -- make the Unique with this
                    UniqSupply UniqSupply
                                 -- when split => these two supplies
 
 mkSplitUniqSupply :: Char -> IO UniqSupply
 -- ^ Create a unique supply out of thin air.
--- The "mask" (Char) supplied is purely cosmetic, making it easier
--- to figure out where a Unique was born. See
--- Note [Uniques and masks].
+-- The "tag" (Char) supplied is mostly cosmetic, making it easier
+-- to figure out where a Unique was born. See Note [Uniques and tags].
 --
 -- The payload part of the Uniques allocated from this UniqSupply are
--- guaranteed distinct wrt all other supplies, regardless of their "mask".
+-- guaranteed distinct wrt all other supplies, regardless of their "tag".
 -- This is achieved by allocating the payload part from
 -- a single source of Uniques, namely `genSym`, shared across
 -- all UniqSupply's.
@@ -210,7 +217,7 @@ mkSplitUniqSupply c
   = unsafeDupableInterleaveIO (IO mk_supply)
 
   where
-     !mask = ord c `unsafeShiftL` uNIQUE_BITS
+     !tag = mkTag c
 
         -- Here comes THE MAGIC: see Note [How the unique supply works]
         -- This is one of the most hammered bits in the whole compiler
@@ -222,43 +229,61 @@ mkSplitUniqSupply c
         -- deferred IO computations
         case unIO (unsafeDupableInterleaveIO (IO mk_supply)) s2 of { (# s3, x #) ->
         case unIO (unsafeDupableInterleaveIO (IO mk_supply)) s3 of { (# s4, y #) ->
-        (# s4, MkSplitUniqSupply (mask .|. u) x y #)
+        (# s4, MkSplitUniqSupply (tag .|. u) x y #)
         }}}}
 
-#if !MIN_VERSION_GLASGOW_HASKELL(9,1,0,0)
-foreign import ccall unsafe "genSym" genSym :: IO Int
+#if defined(NO_FETCH_ADD)
+-- GHC currently does not provide this operation on 32-bit platforms,
+-- hence the CAS-based implementation.
+fetchAddWord64Addr# :: Addr# -> Word64# -> State# RealWorld
+                    -> (# State# RealWorld, Word64# #)
+fetchAddWord64Addr# = go
+  where
+    go ptr inc s0 =
+      case readWord64OffAddr# ptr 0# s0 of
+        (# s1, n0 #) ->
+          case atomicCasWord64Addr# ptr n0 (n0 `plusWord64#` inc) s1 of
+            (# s2, res #)
+              | 1# <- res `eqWord64#` n0 -> (# s2, n0 #)
+              | otherwise -> go ptr inc s2
 #else
-genSym :: IO Int
+fetchAddWord64Addr# :: Addr# -> Word64# -> State# RealWorld
+                    -> (# State# RealWorld, Word64# #)
+fetchAddWord64Addr# addr inc s0 =
+    case fetchAddWordAddr# addr (word64ToWord# inc) s0 of
+      (# s1, res #) -> (# s1, wordToWord64# res #)
+#endif
+
+genSym :: IO Word64
 genSym = do
     let !mask = (1 `unsafeShiftL` uNIQUE_BITS) - 1
-    let !(Ptr counter) = ghc_unique_counter
-    let !(Ptr inc_ptr) = ghc_unique_inc
-    u <- IO $ \s0 -> case readWordOffAddr# inc_ptr 0# s0 of
-        (# s1, inc #) -> case fetchAddWordAddr# counter inc s1 of
+    let !(Ptr counter) = ghc_unique_counter64
+    I# inc# <- peek ghc_unique_inc
+    let !inc = wordToWord64# (int2Word# inc#)
+    u <- IO $ \s1 -> case fetchAddWord64Addr# counter inc s1 of
             (# s2, val #) ->
-                let !u = I# (word2Int# (val `plusWord#` inc)) .&. mask
+                let !u = W64# (val `plusWord64#` inc) .&. mask
                 in (# s2, u #)
 #if defined(DEBUG)
     -- Uh oh! We will overflow next time a unique is requested.
     -- (Note that if the increment isn't 1 we may miss this check)
-    MASSERT(u /= mask)
+    massert (u /= mask)
 #endif
     return u
-#endif
 
-foreign import ccall unsafe "&ghc_unique_counter" ghc_unique_counter :: Ptr Word
-foreign import ccall unsafe "&ghc_unique_inc"     ghc_unique_inc     :: Ptr Int
+foreign import ccall unsafe "&ghc_unique_counter64" ghc_unique_counter64 :: Ptr Word64
+foreign import ccall unsafe "&ghc_unique_inc"       ghc_unique_inc       :: Ptr Int
 
-initUniqSupply :: Word -> Int -> IO ()
+initUniqSupply :: Word64 -> Int -> IO ()
 initUniqSupply counter inc = do
-    poke ghc_unique_counter counter
-    poke ghc_unique_inc     inc
+    poke ghc_unique_counter64 counter
+    poke ghc_unique_inc       inc
 
-uniqFromMask :: Char -> IO Unique
-uniqFromMask !mask
+uniqFromTag :: Char -> IO Unique
+uniqFromTag !tag
   = do { uqNum <- genSym
-       ; return $! mkUnique mask uqNum }
-
+       ; return $! mkUnique tag uqNum }
+{-# NOINLINE uniqFromTag #-} -- We'll unbox everything, but we don't want to inline it
 
 splitUniqSupply :: UniqSupply -> (UniqSupply, UniqSupply)
 -- ^ Build two 'UniqSupply' from a single one, each of which
@@ -279,6 +304,8 @@ uniqFromSupply  (MkSplitUniqSupply n _ _)  = mkUniqueGrimily n
 uniqsFromSupply (MkSplitUniqSupply n _ s2) = mkUniqueGrimily n : uniqsFromSupply s2
 takeUniqFromSupply (MkSplitUniqSupply n s1 _) = (mkUniqueGrimily n, s1)
 
+{-# INLINE splitUniqSupply #-}
+
 {-
 ************************************************************************
 *                                                                      *
@@ -295,12 +322,7 @@ pattern UniqResult x y = (# x, y #)
 
 -- | A monad which just gives the ability to obtain 'Unique's
 newtype UniqSM result = USM { unUSM :: UniqSupply -> UniqResult result }
-
--- See Note [The one-shot state monad trick] for why we don't derive this.
-instance Functor UniqSM where
-  fmap f (USM m) = mkUniqSM $ \us ->
-      case m us of
-        (# r, us' #) -> UniqResult (f r) us'
+  deriving (Functor, Applicative, Monad) via (Strict.State UniqSupply)
 
 -- | Smart constructor for 'UniqSM', as described in Note [The one-shot state
 -- monad trick].
@@ -308,17 +330,6 @@ mkUniqSM :: (UniqSupply -> UniqResult a) -> UniqSM a
 mkUniqSM f = USM (oneShot f)
 {-# INLINE mkUniqSM #-}
 
-instance Monad UniqSM where
-  (>>=) = thenUs
-  (>>)  = (*>)
-
-instance Applicative UniqSM where
-    pure = returnUs
-    (USM f) <*> (USM x) = mkUniqSM $ \us0 -> case f us0 of
-                            UniqResult ff us1 -> case x us1 of
-                              UniqResult xx us2 -> UniqResult (ff xx) us2
-    (*>) = thenUs_
-
 -- TODO: try to get rid of this instance
 instance MonadFail UniqSM where
     fail = panic
@@ -331,30 +342,12 @@ initUs init_us m = case unUSM m init_us of { UniqResult r us -> (r, us) }
 initUs_ :: UniqSupply -> UniqSM a -> a
 initUs_ init_us m = case unUSM m init_us of { UniqResult r _ -> r }
 
-{-# INLINE thenUs #-}
-{-# INLINE returnUs #-}
-{-# INLINE splitUniqSupply #-}
-
--- @thenUs@ is where we split the @UniqSupply@.
-
 liftUSM :: UniqSM a -> UniqSupply -> (a, UniqSupply)
 liftUSM (USM m) us0 = case m us0 of UniqResult a us1 -> (a, us1)
 
 instance MonadFix UniqSM where
     mfix m = mkUniqSM (\us0 -> let (r,us1) = liftUSM (m r) us0 in UniqResult r us1)
 
-thenUs :: UniqSM a -> (a -> UniqSM b) -> UniqSM b
-thenUs (USM expr) cont
-  = mkUniqSM (\us0 -> case (expr us0) of
-                   UniqResult result us1 -> unUSM (cont result) us1)
-
-thenUs_ :: UniqSM a -> UniqSM b -> UniqSM b
-thenUs_ (USM expr) (USM cont)
-  = mkUniqSM (\us0 -> case (expr us0) of { UniqResult _ us1 -> cont us1 })
-
-returnUs :: a -> UniqSM a
-returnUs result = mkUniqSM (\us -> UniqResult result us)
-
 getUs :: UniqSM UniqSupply
 getUs = mkUniqSM (\us0 -> case splitUniqSupply us0 of (us1,us2) -> UniqResult us1 us2)
 
