@@ -106,6 +106,8 @@ stage0Packages = do
              , runGhc
              , semaphoreCompat -- depends on
              , time -- depends on win32
+             , thLift -- new library not yet present for boot compilers
+             , thQuasiquoter -- new library not yet present for boot compilers
              , unlit
              , if windowsHost then win32 else unix
              -- We must use the in-tree `Win32` as the version
