@@ -29,7 +29,7 @@ import Data.List (intersperse, sort)
 import Data.List.NonEmpty (NonEmpty (..))
 import qualified Data.Map as Map
 import qualified Data.Maybe as Maybe
-import GHC hiding (LexicalFixity (..), fromMaybeContext)
+import GHC hiding (HsTypeGhcPsExt (..), LexicalFixity (..), fromMaybeContext)
 import GHC.Core.Type (Specificity (..))
 import GHC.Data.BooleanFormula
 import GHC.Exts hiding (toList)
@@ -82,7 +82,7 @@ ppDecl summ links (L loc decl) pats (mbDoc, fnArgsDoc) instances fixities subdoc
       (locA loc)
       (mbDoc, fnArgsDoc)
       lnames
-      (dropWildCards lty)
+      (dropWildCardsI lty)
       fixities
       splice
       unicode
@@ -335,7 +335,7 @@ ppSubSigLike unicode qual typ argDocs subdocs sep emptyCtxts = do_sig_args 0 sep
       | otherwise =
           (leader <+> ppLContextNoArrow lctxt unicode qual emptyCtxts, Nothing, [])
             : do_largs n (darrow unicode) ltype
-    do_args n leader (HsFunTy _ _w (L _ (HsRecTy _ fields)) r) =
+    do_args n leader (HsFunTy _ _w (L _ (XHsType (HsRecTy fields))) r) =
       [ (ldr <+> html, mdoc, subs)
       | (L _ field, ldr) <- zip fields (leader <+> gadtOpen : repeat gadtComma)
       , let (html, mdoc, subs) = ppSideBySideField subdocs unicode qual field
@@ -469,7 +469,11 @@ ppTySyn
           ( [keyword "type", ppBinder summary occ]
               ++ ppTyVars unicode qual (hsQTvExplicit ltyvars)
           )
-      full = hdr <+> equals <+> ppPatSigType unicode qual (noLocA sig_type)
+      full = hdr <+> def
+      def = case unLoc ltype of
+        XHsType (HsRedacted k) ->
+          dcolon unicode <+> ppType unicode qual HideEmptyContexts k
+        _ -> equals <+> ppPatSigType unicode qual (noLocA sig_type)
       occ = nameOccName . getName $ name
       fixs
         | summary = noHtml
@@ -735,7 +739,7 @@ ppContext cxt unicode qual emptyCtxts = ppContextNoLocs (map unLoc cxt) unicode
 ppHsContext :: [HsType DocNameI] -> Unicode -> Qualification -> Html
 ppHsContext [] _ _ = noHtml
 ppHsContext [p] unicode qual = ppCtxType unicode qual p
-ppHsContext cxt unicode qual = parenList (map (ppType unicode qual HideEmptyContexts) cxt)
+ppHsContext cxt unicode qual = parenBreakableList (map (ppType unicode qual HideEmptyContexts) cxt)
 
 -------------------------------------------------------------------------------
 
@@ -1099,7 +1103,7 @@ ppInstHead links splice unicode qual mdoc origin orphan no ihd@(InstHead{..}) md
               noHtml
               (\t -> equals <+> ppType unicode qual HideEmptyContexts t)
               rhs
-    DataInst dd ->
+    DataInst dd@(DataDecl {}) ->
       ( subInstHead iid pdata
       , mdoc
       , [subFamInstDetails iid pdecl mname]
@@ -1109,6 +1113,7 @@ ppInstHead links splice unicode qual mdoc origin orphan no ihd@(InstHead{..}) md
         pref = case cons of NewTypeCon _ -> keyword "newtype"; DataTypeCons _ _ -> keyword "data"
         pdata = pref <+> typ
         pdecl = pdata <+> ppShortDataDecl False True dd [] unicode qual
+    DataInst {} -> error "ppInstHead"
   where
     mname = maybe noHtml (\m -> toHtml "Defined in" <+> ppModule m) mdl
     iid = instanceId origin no orphan ihd
@@ -1146,7 +1151,7 @@ ppInstanceSigs
 ppInstanceSigs links splice unicode qual sigs = do
   TypeSig _ lnames typ <- sigs
   let names = map unLoc lnames
-      L _ rtyp = dropWildCards typ
+      L _ rtyp = dropWildCardsI typ
   -- Instance methods signatures are synified and thus don't have a useful
   -- SrcSpan value. Use the methods name location instead.
   let lname =
@@ -1187,7 +1192,7 @@ ppShortDataDecl
   -> Unicode
   -> Qualification
   -> Html
-ppShortDataDecl summary dataInst dataDecl pats unicode qual
+ppShortDataDecl summary dataInst dataDecl@(DataDecl {}) pats unicode qual
   | [] <- toList cons
   , [] <- pats =
       dataHeader
@@ -1224,6 +1229,7 @@ ppShortDataDecl summary dataInst dataDecl pats unicode qual
         ]
       | (SigD _ (PatSynSig _ lnames typ), _) <- pats
       ]
+ppShortDataDecl _ _ _ _ _ _ = error "ppShortDataDecl"
 
 -- | Pretty-print a data declaration
 ppDataDecl
@@ -1239,7 +1245,7 @@ ppDataDecl
   -> Documentation DocName
   -- ^ this decl's documentation
   -> TyClDecl DocNameI
-  -- ^ this decl
+  -- ^ this decl; always a DataDecl
   -> [(HsDecl DocNameI, DocForDecl DocName)]
   -- ^ relevant patterns
   -> Splice
@@ -1255,7 +1261,7 @@ ppDataDecl
   subdocs
   loc
   doc
-  dataDecl
+  dataDecl@(DataDecl {})
   pats
   splice
   unicode
@@ -1325,6 +1331,7 @@ ppDataDecl
           unicode
           pkg
           qual
+ppDataDecl _ _ _ _ _ _ _ _ _ _ _ _ _ = error "ppDataDecl"
 
 ppShortConstr :: Bool -> ConDecl DocNameI -> Unicode -> Qualification -> Html
 ppShortConstr summary con unicode qual = cHead <+> cBody <+> cFoot
@@ -1346,8 +1353,8 @@ ppShortConstrParts summary dataInst con unicode qual =
             header_ = ppConstrHdr forall_ tyVars context unicode qual
          in case det of
               -- Prefix constructor, e.g. 'Just a'
-              PrefixCon _ args ->
-                ( header_ <+> hsep (ppOcc : map (ppLParendType unicode qual HideEmptyContexts . hsScaledThing) args)
+              PrefixCon args ->
+                ( header_ <+> hsep (ppOcc : map (ppLParendType unicode qual HideEmptyContexts . hsConDeclFieldToHsTypeNoMult) args)
                 , noHtml
                 , noHtml
                 )
@@ -1365,9 +1372,9 @@ ppShortConstrParts summary dataInst con unicode qual =
               InfixCon arg1 arg2 ->
                 ( header_
                     <+> hsep
-                      [ ppLParendType unicode qual HideEmptyContexts (hsScaledThing arg1)
+                      [ ppLParendType unicode qual HideEmptyContexts (hsConDeclFieldToHsTypeNoMult arg1)
                       , ppOccInfix
-                      , ppLParendType unicode qual HideEmptyContexts (hsScaledThing arg2)
+                      , ppLParendType unicode qual HideEmptyContexts (hsConDeclFieldToHsTypeNoMult arg2)
                       ]
                 , noHtml
                 , noHtml
@@ -1423,12 +1430,12 @@ ppSideBySideConstr subdocs fixities unicode pkg qual (L _ con) =
               header_ = ppConstrHdr forall_ tyVars context unicode qual
            in case det of
                 -- Prefix constructor, e.g. 'Just a'
-                PrefixCon _ args
+                PrefixCon args
                   | hasArgDocs -> header_ <+> ppOcc <+> fixity
                   | otherwise ->
                       hsep
                         [ header_ <+> ppOcc
-                        , hsep (map (ppLParendType unicode qual HideEmptyContexts . hsScaledThing) args)
+                        , hsep (map (ppLParendType unicode qual HideEmptyContexts . hsConDeclFieldToHsTypeNoMult) args)
                         , fixity
                         ]
                 -- Record constructor, e.g. 'Identity { runIdentity :: a }'
@@ -1438,9 +1445,9 @@ ppSideBySideConstr subdocs fixities unicode pkg qual (L _ con) =
                   | hasArgDocs -> header_ <+> ppOcc <+> fixity
                   | otherwise ->
                       hsep
-                        [ header_ <+> ppLParendType unicode qual HideEmptyContexts (hsScaledThing arg1)
+                        [ header_ <+> ppLParendType unicode qual HideEmptyContexts (hsConDeclFieldToHsTypeNoMult arg1)
                         , ppOccInfix
-                        , ppLParendType unicode qual HideEmptyContexts (hsScaledThing arg2)
+                        , ppLParendType unicode qual HideEmptyContexts (hsConDeclFieldToHsTypeNoMult arg2)
                         , fixity
                         ]
       -- GADT constructor, e.g. 'Foo :: Int -> Foo'
@@ -1466,7 +1473,7 @@ ppSideBySideConstr subdocs fixities unicode pkg qual (L _ con) =
         -- H98 record declarations
         RecCon (L _ fields) -> [doRecordFields fields]
         -- H98 prefix data constructors
-        PrefixCon _ args | hasArgDocs -> [doConstrArgsWithDocs args]
+        PrefixCon args | hasArgDocs -> [doConstrArgsWithDocs args]
         -- H98 infix data constructor
         InfixCon arg1 arg2 | hasArgDocs -> [doConstrArgsWithDocs [arg1, arg2]]
         _ -> []
@@ -1477,10 +1484,11 @@ ppSideBySideConstr subdocs fixities unicode pkg qual (L _ con) =
         qual
         (map (ppSideBySideField subdocs unicode qual) (map unLoc fields))
 
+    doConstrArgsWithDocs :: [HsConDeclField DocNameI] -> Html
     doConstrArgsWithDocs args = subFields pkg qual $ case con of
       ConDeclH98{} ->
         [ (ppLParendType unicode qual HideEmptyContexts arg, mdoc, [])
-        | (i, arg) <- zip [0 ..] (map hsScaledThing args)
+        | (i, arg) <- zip [0 ..] (map hsConDeclFieldToHsTypeNoMult args)
         , let mdoc = Map.lookup i argDocs
         ]
       ConDeclGADT{} ->
@@ -1528,9 +1536,9 @@ ppSideBySideField
   :: [(DocName, DocForDecl DocName)]
   -> Unicode
   -> Qualification
-  -> ConDeclField DocNameI
+  -> HsConDeclRecField DocNameI
   -> SubDecl
-ppSideBySideField subdocs unicode qual (ConDeclField _ names ltype _) =
+ppSideBySideField subdocs unicode qual (HsConDeclRecField _ names ltype) =
   ( hsep
       ( punctuate
           comma
@@ -1539,24 +1547,32 @@ ppSideBySideField subdocs unicode qual (ConDeclField _ names ltype _) =
           , let field = (foExt) name
           ]
       )
+      <+> ppRecFieldMultAnn unicode qual ltype
       <+> dcolon unicode
-      <+> ppLType unicode qual HideEmptyContexts ltype
+      <+> ppLType unicode qual HideEmptyContexts (hsConDeclFieldToHsTypeNoMult ltype)
   , mbDoc
   , []
   )
   where
-    -- don't use cd_fld_doc for same reason we don't use con_doc above
-    -- Where there is more than one name, they all have the same documentation
     mbDoc = lookup (unLoc . foLabel $ unLoc declName) subdocs >>= combineDocumentation . fst
     declName = case Maybe.listToMaybe names of
       Nothing -> error "No names. An invariant was broken. Please report this to the Haddock project"
       Just hd -> hd
 
-ppShortField :: Bool -> Unicode -> Qualification -> ConDeclField DocNameI -> Html
-ppShortField summary unicode qual (ConDeclField _ names ltype _) =
+-- don't use cdf_doc for same reason we don't use con_doc above
+-- Where there is more than one name, they all have the same documentation
+ppRecFieldMultAnn :: Unicode -> Qualification -> HsConDeclField DocNameI -> Html
+ppRecFieldMultAnn unicode qual (CDF { cdf_multiplicity = ann }) = case ann of
+  HsUnannotated _ -> noHtml
+  HsLinearAnn _ -> toHtml "%1"
+  HsExplicitMult _ mult -> multAnnotation <> ppr_mono_lty mult unicode qual HideEmptyContexts
+
+ppShortField :: Bool -> Unicode -> Qualification -> HsConDeclRecField DocNameI -> Html
+ppShortField summary unicode qual (HsConDeclRecField _ names ltype) =
   hsep (punctuate comma (map ((ppBinder summary) . rdrNameOcc . foExt . unLoc) names))
+    <+> ppRecFieldMultAnn unicode qual ltype
     <+> dcolon unicode
-    <+> ppLType unicode qual HideEmptyContexts ltype
+    <+> ppLType unicode qual HideEmptyContexts (hsConDeclFieldToHsTypeNoMult ltype)
 
 -- | Pretty print an expanded pattern (for bundled patterns)
 ppSideBySidePat
@@ -1651,9 +1667,9 @@ ppDataHeader _ _ _ _ = error "ppDataHeader: illegal argument"
 
 --------------------------------------------------------------------------------
 
-ppBang :: HsBang -> Html
-ppBang (HsBang _ SrcStrict) = toHtml "!"
-ppBang (HsBang _ SrcLazy) = toHtml "~"
+ppBang :: HsSrcBang -> Html
+ppBang (HsSrcBang _ _ SrcStrict) = toHtml "!"
+ppBang (HsSrcBang _ _ SrcLazy) = toHtml "~"
 ppBang _ = noHtml
 
 tupleParens :: HsTupleSort -> [Html] -> Html
@@ -1799,8 +1815,6 @@ ppr_mono_ty (HsQualTy _ ctxt ty) unicode qual emptyCtxts =
 -- UnicodeSyntax alternatives
 ppr_mono_ty (HsTyVar _ _ (L _ name)) True _ _
   | getOccString (getName name) == "(->)" = toHtml "(→)"
-ppr_mono_ty (HsBangTy _ b ty) u q _ =
-  ppBang b +++ ppLParendType u q HideEmptyContexts ty
 ppr_mono_ty (HsTyVar _ prom (L _ name)) _ q _
   | isPromoted prom = promoQuote (ppDocName q Prefix True name)
   | otherwise = ppDocName q Prefix True name
@@ -1813,8 +1827,8 @@ ppr_mono_ty (HsFunTy _ mult ty1 ty2) u q e =
     ]
   where
     arr = case mult of
-      HsLinearArrow _ -> lollipop u
-      HsUnrestrictedArrow _ -> arrow u
+      HsLinearAnn _ -> lollipop u
+      HsUnannotated _ -> arrow u
       HsExplicitMult _ m -> multAnnotation <> ppr_mono_lty m u q e <+> arrow u
 ppr_mono_ty (HsTupleTy _ con tys) u q _ =
   tupleParens con (map (ppLType u q HideEmptyContexts) tys)
@@ -1826,11 +1840,13 @@ ppr_mono_ty (HsListTy _ ty) u q _ = brackets (ppr_mono_lty ty u q HideEmptyConte
 ppr_mono_ty (HsIParamTy _ (L _ n) ty) u q _ =
   ppIPName n <+> dcolon u <+> ppr_mono_lty ty u q HideEmptyContexts
 ppr_mono_ty (HsSpliceTy v _) _ _ _ = dataConCantHappen v
-ppr_mono_ty (HsRecTy{}) _ _ _ = toHtml "{..}"
+ppr_mono_ty (XHsType (HsBangTy b ty)) u q _ =
+  ppBang b +++ ppLParendType u q HideEmptyContexts ty
+ppr_mono_ty (XHsType (HsRecTy{})) _ _ _ = toHtml "{..}"
 -- Can now legally occur in ConDeclGADT, the output here is to provide a
 -- placeholder in the signature, which is followed by the field
 -- declarations.
-ppr_mono_ty (XHsType{}) _ _ _ = error "ppr_mono_ty HsCoreTy"
+ppr_mono_ty (XHsType HsCoreTy{}) _ _ _ = error "ppr_mono_ty HsCoreTy"
 ppr_mono_ty (HsExplicitListTy _ IsPromoted tys) u q _ = promoQuote $ brackets $ hsep $ punctuate comma $ map (ppLType u q HideEmptyContexts) tys
 ppr_mono_ty (HsExplicitListTy _ NotPromoted tys) u q _ = brackets $ hsep $ punctuate comma $ map (ppLType u q HideEmptyContexts) tys
 ppr_mono_ty (HsExplicitTupleTy _ IsPromoted tys) u q _ = promoQuote $ parenList $ map (ppLType u q HideEmptyContexts) tys
@@ -1862,6 +1878,7 @@ ppr_mono_ty (HsDocTy _ ty _) unicode qual emptyCtxts =
   ppr_mono_lty ty unicode qual emptyCtxts
 ppr_mono_ty (HsWildCardTy _) _ _ _ = char '_'
 ppr_mono_ty (HsTyLit _ n) _ _ _ = ppr_tylit n
+ppr_mono_ty (XHsType HsRedacted{}) _ _ _ = error "ppr_mono_ty: HsRedacted can't be used here"
 
 ppr_tylit :: HsTyLit DocNameI -> Html
 ppr_tylit (HsNumTy _ n) = toHtml (show n)
